/* 2004-12-12 SMS.
 *
 * Emergency replacement UNIXLIB.H for GNU C, for use as needed.
 * Install as GNU_CC_INCLUDE:[000000]UNIXLIB.H
 */

#ifndef __UNIXLIB_LOADED
#define __UNIXLIB_LOADED 1

#include <sys/types.h>
#include <stdlib.h>

typedef struct stat stat_t;

#endif  /* ndef __UNIXLIB_LOADED */

